﻿Type=Class
Version=3
@EndOfDesignText@
'Class module
' Name:		clsFilesLists
' Version: 	1.0
' Date:		02/02/2014
' Author:	LucaMs
'
Sub Class_Globals
	Type typFilter(NameFilter As String, NamePosition As Int, ExtFilter As String, ExtPosition As Int)

	' CONSTANTS
	Private F_STARTSWITH As Int = 1
	Private F_ENDSWITH As Int = 2
	Private F_CONTAINS As Int = 3
	Private F_NONE As Int = 4
End Sub

' With this class you can:
'
' get an ordered list of files from a directory without directories names;
' get an ordered list of files from a directory filtered using classic pattern;
' delete all files in a directory that match a classic pattern.
' (patterns like: "MyFile*.txt" or "*My*.*" or "Image.jp*" etc.).
'
' You should use this class almost as if it were a Singleton class: one instance only.
Public Sub Initialize
End Sub


#Region  METHODS 

#Region  Public 

' Gets a filtered list of files (without subdirectories)
' Dir		: the directory from which you want files
' Pattern: classic filter pattern (like: "MyFile*.txt" or "*My*.*" or "Image.jp*" etc.)
Public Sub ListFiles(Dir As String, Pattern As String, Sorted As Boolean, Ascending As Boolean) As List	
	Dim lstFiles As List
	lstFiles.Initialize
	lstFiles = GetFilteredFileList(Dir, Pattern, Sorted, Ascending)
	
	Return lstFiles
End Sub

' Deletes all files from a directory that match a given pattern.
' It does not deletes subdirectory files.
' Dir		: the directory from which you want delete files
' Pattern: classic filter pattern (like: "MyFile*.txt" or "*My*.*", ...)
Public Sub DeleteFiles(Dir As String, Pattern As String)
	Dim lstFilesToDelete As List
	lstFilesToDelete.Initialize
	lstFilesToDelete = GetFilteredFileList(Dir, Pattern, False, False)
	
	For i = 0 To lstFilesToDelete.Size - 1
		File.Delete(Dir, lstFilesToDelete.Get(i))
	Next
End Sub

' Similar to File.ListFiles(Dir As String) but returns only file names, excluding directories,
' and the list can be sorted.
Public Sub ListFilesOnly(Dir As String, Sorted As Boolean, Ascending As Boolean) As List
	Dim lstDir, lstRes As List
	
	If File.Exists(Dir, "") Then

		lstDir = File.ListFiles(Dir)
		lstRes.Initialize
		
		Dim FileName As String
		
		For i = 0 To lstDir.Size - 1
			FileName = lstDir.Get(i)
			If Not(File.IsDirectory(Dir, FileName)) Then
				lstRes.Add(FileName)
			End If
		Next
		
	End If
	
	If Sorted Then
		lstRes.Sort(Ascending)
	End If
	
	Return lstRes

End Sub

#End Region

#Region  Private 

Private Sub GetFilteredFileList(Dir As String, Pattern As String, Sorted As Boolean, Ascending As Boolean) As List
	If Pattern = "*.*" Then
		Return ListFilesOnly(Dir, Sorted, Ascending)
	End If

	Dim lstRes As List
	lstRes.Initialize

	' Get the files list in the Dir directory without directories names.
	Dim lstFilesOnly As List
	lstFilesOnly = ListFilesOnly(Dir, Sorted, Ascending)
	
	' Analyzes the Pattern
	Dim Filter As typFilter
	Filter.Initialize
	Filter = ParsePattern(Pattern)
	
	' Filters the files list
	Dim FileName As String
	Dim FileBaseName As String
	Dim Extension As String
	Dim Valid As Boolean
	
	For i = 0 To lstFilesOnly.Size - 1
		FileName = lstFilesOnly.Get(i)
		FileBaseName = FileName.SubString2(0, FileName.LastIndexOf(".")) 
		' Filters on name
		Select Case Filter.NamePosition
			Case F_NONE
				Valid = True
			Case F_STARTSWITH
				Valid = FileBaseName.StartsWith(Filter.NameFilter)
			Case F_ENDSWITH
				Valid = FileBaseName.EndsWith(Filter.NameFilter)
			Case F_CONTAINS
				Valid = FileBaseName.Contains(Filter.NameFilter)
		End Select
		
		If Valid AND Filter.ExtFilter <> "*" Then
			' Filters on extension
			Extension = FileName.SubString(FileName.LastIndexOf(".") + 1) 
			Select Case Filter.ExtPosition
				Case F_NONE
					Valid = True
				Case F_STARTSWITH
					Valid = Extension.StartsWith(Filter.ExtFilter)
				Case F_ENDSWITH
					Valid = Extension.EndsWith(Filter.ExtFilter)
				Case F_CONTAINS
					Valid = Extension.Contains(Filter.ExtFilter)
			End Select
		End If

		If Valid Then
			lstRes.Add(FileName)
		End If
		
	Next
	
	Return lstRes
	
End Sub

Private Sub ParsePattern(Pattern As String) As typFilter
	Dim tFilter As typFilter
	tFilter.Initialize
	
	Dim Parts() As String
	
	Parts = Regex.Split("\.", Pattern)
	
	If Parts.Length > 2 Then
		Log("error")
		Return tFilter
	End If
	
	Dim FileNameFilter As String = Parts(0)
	Dim ExtFilter As String
	If Parts.Length > 1 Then
		ExtFilter = Parts(1)
	End If
	
	' Analyzes the file name pattern
	If FileNameFilter = "*" Then
		tFilter.NamePosition = F_NONE
	Else
		If FileNameFilter.StartsWith("*") Then
			If FileNameFilter.EndsWith("*") Then
				tFilter.NamePosition = F_CONTAINS
			Else
				tFilter.NamePosition = F_ENDSWITH
			End If
		Else
			If FileNameFilter.EndsWith("*") Then
				tFilter.NamePosition = F_STARTSWITH
			End If
		End If
	End If
	
	If FileNameFilter <> "*" Then
		tFilter.NameFilter = FileNameFilter.Replace("*", "")
	Else
		tFilter.NameFilter = "*"
	End If
	
	
	' Analyzes the extension pattern
	If ExtFilter = "*" Then
		tFilter.ExtPosition = F_NONE
	Else
		If ExtFilter.StartsWith("*") Then
			If ExtFilter.EndsWith("*") Then
				tFilter.ExtPosition = F_CONTAINS
			Else
				tFilter.ExtPosition = F_ENDSWITH
			End If
		Else
			If ExtFilter.EndsWith("*") Then
				tFilter.ExtPosition = F_STARTSWITH
			End If
		End If
	End If

	If ExtFilter <> "*" Then
		tFilter.ExtFilter = ExtFilter.Replace("*", "")
	Else
		tFilter.ExtFilter = "*"
	End If
	
	Return tFilter

End Sub

#End Region

#End Region